"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const create_single_rhino_state_1 = require("./create-single-rhino-state");
/**
 *
 * Function to creates global state using rhino
 * @param globalStatesObject An Object containing all the global states to be created
 * @example <caption>Sample Use of createRhinoState</caption>
 * const { Provider, useRhinoState }=createRhinoState({user:"John Doe",isLoggedIn:true});
 */
function createRhinoState(globalStatesObject) {
    const stateDataList = {};
    Object.keys(globalStatesObject).forEach((key) => {
        const initialValue = globalStatesObject[key];
        const { Provider, useStateValue, useStateUpdate } = create_single_rhino_state_1.default(initialValue);
        stateDataList[key] = {
            provider: Provider,
            stateHook: useStateValue,
            updaterHook: useStateUpdate,
        };
    });
    /**
     *
     * Provider Component which makes the states available to all the nested components
     */
    function RhinoProvider({ children }) {
        const ProviderList = Object.values(stateDataList).map((stateData) => stateData.provider);
        // Combining all providers into a single provider
        const RootProvider = ProviderList.reduce((ProviderAccumulator, Provider) => {
            return ({ children }) => (React.createElement(ProviderAccumulator, null,
                React.createElement(Provider, null, children)));
        });
        return React.createElement(RootProvider, null, children);
    }
    /**
     *
     * React hook which returns the stateful value
     * corresponding to the key provided
     * @param key Key which represents the state
     */
    function useRhinoValue(key) {
        if (key in stateDataList) {
            return stateDataList[key].stateHook();
        }
        throw new Error(`${key} is an invalid key`);
    }
    /**
     *
     * React hook which returns a function to update the global state
     * corresponding to the key provided
     * @param key Key which represents the state
     */
    function useSetRhinoState(key) {
        if (key in stateDataList) {
            return stateDataList[key].updaterHook();
        }
        throw new Error(`${key} is an invalid key`);
    }
    /**
     *
     * React hook which returns the stateful value  corresponding to the key
     * provided and a function to update it
     * @param key Key which represents the state
     */
    function useRhinoState(key) {
        if (key in stateDataList) {
            return [useRhinoValue(key), useSetRhinoState(key)];
        }
        throw new Error(`${key} is an invalid key`);
    }
    return {
        RhinoProvider,
        useRhinoState,
        useRhinoValue,
        useSetRhinoState,
    };
}
exports.default = createRhinoState;
